@@ -10,6 +10,7 @@
 #include "config.hh"
 #include "csvreader.hh"
 #include "dfufile.hh"
+#include "radiolimits.hh"
 #include "rd5r.hh"
 #include "uv390.hh"
 #include "md2017.hh"
@@ -67,45 +68,47 @@ int verify(QCommandLineParser &parser, QCoreApplication &app)
     return 0;
   }
 
-  QList<VerifyIssue> issues;
+  RadioLimitContext ctx;
   QString radio = parser.value("radio").toLower();
   if ("rd5r" == radio) {
-    RD5R radio; radio.verifyConfig(&config, issues);
+    RD5R radio; radio.limits().verifyConfig(&config, ctx);
   } else if (("uv390" == radio) || ("rt3s" == radio)) {
-    UV390 radio; radio.verifyConfig(&config, issues);
+    UV390 radio; radio.limits().verifyConfig(&config, ctx);
   } else if (("md2017" == radio) || ("rt82" == radio)) {
-    MD2017 radio; radio.verifyConfig(&config, issues);
+    MD2017 radio; radio.limits().verifyConfig(&config, ctx);
   } else if ("gd77" == radio) {
-    GD77 radio; radio.verifyConfig(&config, issues);
+    GD77 radio; radio.limits().verifyConfig(&config, ctx);
   } else if ("opengd77" == radio) {
-    OpenGD77 radio; radio.verifyConfig(&config, issues);
+    OpenGD77 radio; radio.limits().verifyConfig(&config, ctx);
   } else if ("d868uv" == radio) {
-    D868UV radio; radio.verifyConfig(&config, issues);
+    D868UV radio; radio.limits().verifyConfig(&config, ctx);
   } else if ("d878uv" == radio) {
-    D878UV radio; radio.verifyConfig(&config, issues);
+    D878UV radio; radio.limits().verifyConfig(&config, ctx);
   } else if ("d878uv2" == radio) {
-    D878UV2 radio; radio.verifyConfig(&config, issues);
+    D878UV2 radio; radio.limits().verifyConfig(&config, ctx);
   } else if ("d578uv" == radio) {
-    D578UV radio; radio.verifyConfig(&config, issues);
+    D578UV radio; radio.limits().verifyConfig(&config, ctx);
   } else {
     logError() << "Cannot verify code-plug against unknown radio '" << radio << "'.";
     return -1;
   }
 
   bool valid = true;
-  if (0 == issues.size()) {
-    logInfo() << "No issues found for code-plug '" << filename
-              << "' with radio '" << radio << "'.";
-  } else {
-    logInfo() << "Some issues found for code-plug '" << filename
-              << "' with radio '" << radio << "'.";
-    foreach (const VerifyIssue &issue, issues) {
-      if (VerifyIssue::WARNING == issue.type()) {
-        logWarn() << issue.message();
-      } else if (VerifyIssue::ERROR == issue.type()) {
-        logError() << issue.message();
-        valid = false;
-      }
+  for (int i=0; i<ctx.count(); i++) {
+    switch (ctx.message(i).severity()) {
+    case RadioLimitIssue::Silent:
+      logDebug() << ctx.message(i).format();
+      break;
+    case RadioLimitIssue::Hint:
+      logInfo() << ctx.message(i).format();
+      break;
+    case RadioLimitIssue::Warning:
+      logWarn() << ctx.message(i).format();
+      break;
+    case RadioLimitIssue::Critical:
+      logError() << ctx.message(i).format();
+      valid = false;
+      break;
     }
   }
 
