@@ -21,6 +21,9 @@
 #define CHANNEL_BITMAP            0x024c1500
 #define CHANNEL_BITMAP_SIZE       0x00000200
 
+#define ADDR_HIDDEN_ZONE_MAP      0x024c1360
+#define HIDDEN_ZONE_MAP_SIZE      0x00000020
+
 #define ADDR_GENERAL_CONFIG       0x02500000
 #define GENERAL_CONFIG_SIZE       0x00000100
 
@@ -1365,6 +1368,14 @@ D878UVCodeplug::AnalogAPRSSettingsElement::clear() {
   setUInt8(0x003d, 0x01); setUInt8(0x003e, 0x03); setUInt8(0x003f, 0xff);
 }
 
+bool
+D878UVCodeplug::AnalogAPRSSettingsElement::isValid() const {
+  if (! Codeplug::Element::isValid())
+    return false;
+  return (0 != frequency()) && (! destination().simplified().isEmpty())
+      && (! source().simplified().isEmpty());
+}
+
 unsigned
 D878UVCodeplug::AnalogAPRSSettingsElement::frequency() const {
   return ((unsigned)getBCD8_be(0x0001))*10;
@@ -2278,6 +2289,52 @@ D878UVCodeplug::linkChannels(Context &ctx, const ErrorStack &err) {
 }
 
 
+void
+D878UVCodeplug::allocateZones() {
+  D868UVCodeplug::allocateZones();
+  // Hidden zone map
+  image(0).addElement(ADDR_HIDDEN_ZONE_MAP, HIDDEN_ZONE_MAP_SIZE);
+}
+
+bool
+D878UVCodeplug::encodeZone(int i, Zone *zone, bool isB, const Flags &flags, Context &ctx, const ErrorStack &err) {
+  if (! D868UVCodeplug::encodeZone(i, zone, isB, flags, ctx, err))
+    return false;
+
+  AnytoneZoneExtension *ext = zone->anytoneExtension();
+  if (nullptr == ext)
+      return true;
+
+  if (ext->hidden()) {
+    // set bit
+    data(ADDR_HIDDEN_ZONE_MAP)[i/8] |= (1<<(i%8));
+  } else {
+    // clear bit
+    data(ADDR_HIDDEN_ZONE_MAP)[i/8] &= ~(1<<(i%8));
+  }
+
+  return true;
+}
+
+bool
+D878UVCodeplug::decodeZone(int i, Zone *zone, bool isB, Context &ctx, const ErrorStack &err) {
+  if (! D868UVCodeplug::decodeZone(i, zone, isB, ctx, err))
+    return false;
+  AnytoneZoneExtension *ext = zone->anytoneExtension();
+  if (nullptr == ext) {
+    ext = new AnytoneZoneExtension();
+    zone->setAnytoneExtension(ext);
+  }
+
+  if ((! isB) && (data(ADDR_HIDDEN_ZONE_MAP)[i/8] & (1<<(i%8))))
+    ext->enableHidden(true);
+  else
+    ext->enableHidden(false);
+
+  return true;
+}
+
+
 void
 D878UVCodeplug::allocateGeneralSettings() {
   // override allocation of general settings for D878UV code-plug. General settings are larger!
